/*
 * lab1.c
*/
#include <avr/io.h>
#include <avr/interrupt.h>

#define F_CPU 16000000

#include <string.h>
#include <stdio.h>
#include <stdbool.h>

#include "lcd.h"
#include "usart.h"
#include "pff.h"
#include "spi.h"
#include "sd.h"

#include <avr/io.h>
#include <util/delay.h>

#define MUSIC "music"
#include "pff.h"

FATFS fs;					// sistemul de fisiere

//************************************************************************************//
//                                         ADC
//************************************************************************************//

#define ADC_USE_IRQ

/* Tensiunea de referinta utilizata de ADC. */
#define ADC_AREF_VOLTAGE 3.3
#define ADC_MAX_CHANNELS 6

/* vector cu valorile senzorilor */
uint16_t SensorsVals[5];

/* Numele canalelor de ADC. */
typedef struct
{
	const char *name;
	uint8_t    channel;
} adc_channel_t;

/* Perechi de (Nume canal, Numar canal). */
const adc_channel_t ADC_channels[] = {
	{"Senzor1", 0},
    {"Senzor2", 1},
	{"Senzor3", 2},
	{"Senzor4", 3},
	{"Senzor5", 4},
};

/* Numarul de canale ADC utilizate. */
const int adc_num_channels = sizeof(ADC_channels) / sizeof(ADC_channels[0]);

#ifdef ADC_USE_IRQ
/* Numarul canalului initial. */
#define ADC_INIT_CHANNEL 0

/* Valorile citite de la fiecare canal. */
volatile uint16_t ADC_value[ADC_MAX_CHANNELS];
#endif

#ifdef ADC_USE_IRQ
ISR(ADC_vect)
{
    // Intreruperea este triggered cand o conversie este finalizata. */

    // TODO 4
    // * store new result
    // * change channel
    // * start a new conversion if there are any channels left
    static uint8_t channel = ADC_INIT_CHANNEL;

    ADC_value[channel] = ADC;

    // Canalele utilizate nu sunt consecutive!

    channel = (channel + 1) % ADC_MAX_CHANNELS;

    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    // Pornim ADC-ul daca le-am citit pe toate.
    if (channel != ADC_INIT_CHANNEL)
        ADCSRA |= (1 << ADSC);
}

ISR(TIMER2_COMPA_vect)
{
	// TODO 4
	// start an ADC conversion
	ADCSRA |= (1 << ADSC);
}
#endif


/*
 * Functia initializeaza convertorul Analog-Digital.
 */
void ADC_init(void)
{
	#ifndef ADC_USE_IRQ
	// TODO 2
	// enable ADC with:
	// * reference AVCC with external capacitor at AREF pin
	// * without left adjust of conversion result
	// * no auto-trigger
	// * no interrupt
	// * prescaler at 128
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
	
	#else
	// TODO 4
	// enable ADC with interrupt, the rest of the settings like in TODO 2
	ADMUX = (1 << REFS0) | (1 << ADC_INIT_CHANNEL);
	ADCSRA = (1 << ADEN) | (1 << ADIE) | (7 << ADPS0);
	#endif
}

uint16_t ADC_get(uint8_t channel)
{
	#ifndef ADC_USE_IRQ
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while((ADCSRA & (1 << ADIF)) == 0);

	return ADC;
	

	#else
	//TODO 4
	return ADC_value[channel];

	#endif

(void)channel;
return 0; 
}

/*
 * Functia afiseaza pe display-ul grafic numele canalelor ADC si valorile
 * citite de la acestea.
 */
void ADC_show(void)
{
		
		LCD_clear_bottom_line();
		LCD_printAt(0x40, "show");
    uint16_t a;

//  LCD_clear_top_line();
//  LCD_printAt(0x00, "Senzor1: ");
	for(int i= 0; i<adc_num_channels; i++)
	{
		const adc_channel_t *channel = &ADC_channels[i];
		LCD_clear_top_line();
		LCD_printAt(0x00, channel->name);
		a = ADC_get(channel->channel);
		SensorsVals[i] = a;
		
		int m = SensorsVals[i];
		char arr[200], arr2[200];
		sprintf(arr, "da: %d", m);
		sprintf(arr2, "da: %d", a);
		LCD_print(arr);
		LCD_printAt(0x40, arr2);
			
		_delay_ms(1000);
		
	}
		
}


#ifdef ADC_USE_IRQ
void TIMER2_init()
{
	// TODO 4
	// initialize timer to trigger COMPA every 100ms
	TCCR2A |= (1 << WGM21);
	TCCR2B |= (7 << CS20);
	TIMSK2 |= (1 << OCIE2A);
	OCR2A = 1561;
}
#endif

//******************************************************************//
//                            FINAL ADC  
//******************************************************************//

void IO_init(void)
{	
	DDRD &= ~(1 << PD6);                    // PD6 ca input, cu pull-up
	PORTD |= (1 << PD6);
	
	DDRD |= (1 << PD0);
	DDRD |= (1 << PD1);
		
}

#define FCC(c1, c2, c3, c4) \
	(((DWORD)(c4) << 24) + \
	 ((DWORD)(c3) << 16) + \
	 (( WORD)(c2) <<  8) + \
	 (( BYTE)(c1) <<  0))


uint8_t	buf[2][256];	// wave output buffers (double buffering)
const	 uint16_t	buf_size = 256;	// front and back buffer sizes
volatile uint8_t	buf_front = 0;	// front buffer index (current buffer used)
volatile uint8_t	buf_pos = 0;	// current buffer position
volatile uint8_t	buf_sync = 0;

#define BUF_FRONT	(buf[buf_front])
#define BUF_BACK	(buf[1 - buf_front])

char current_file_no = 1;
char filename[32] = { 0 };

ISR(TIMER0_COMPA_vect)
{
	
	OCR1B = BUF_FRONT[buf_pos++];
	
	// swap buffers when end is reached (end is 256 <=> overflow to 0)
	if(buf_pos == 0)
		buf_front = 1 - buf_front;
	//	PORTD ^= (1 << PD1);
		
}

void timer0_start(void)
{
	// interrupt on compare A
	TIMSK0 |= (1 << OCIE0A);
	// CTC, top OCRA
	TCCR0B |= (0 << WGM02);
	TCCR0A |= (1 << WGM01) | (0 << WGM00);
	// prescaler 8
	TCCR0B |= (2 << CS00);
}

void timer0_stop(void)
{
	TCCR0B = 0;
	TCCR0A = 0;
	TIMSK0 = 0;
	OCR0A = 0;
	TCNT0 = 0;
}

void timer1_start(void)
{
	// 8-bit FastPWM
	TCCR1B |= (1 << WGM12);
	TCCR1A |= (1 << WGM10);
	// channel B inverted
	TCCR1A |= (1 << COM1B0) | (1 << COM1B1);
	// prescaler 1
	TCCR1B |= (1 << CS10);
}

void timer1_stop(void)
{
	TCCR1B = 0;
	TCCR1A = 0;
	OCR1B = 0;
	TCNT1 = 0;
}

bool continue_play()
{
	if((PINB & (1 << PB6)) == 0)
		return false;
	else
	return true;
}

/*
 * Incarca header-ul unui fisier WAVE
 *
 * @return DWORD
 * 	0 => format invalid
 * 	1 => eroare I/O
 * 	>1 => numarul de sample-uri
 */
DWORD load_header(void)
{
	DWORD size;
	WORD ret;

	// citeste header-ul (12 octeti)
	if(pf_read(BUF_FRONT, 12, &ret))
		return 1;

	if(ret != 12 || LD_DWORD(BUF_FRONT + 8) != FCC('W','A','V','E'))
		return 0;

	for(;;)
	{
		// citeste chunk ID si size
		pf_read(BUF_FRONT, 8, &ret);
		if(ret != 8)
			return 0;

		size = LD_DWORD(&BUF_FRONT[4]);

		// verifica FCC
		switch(LD_DWORD(&BUF_FRONT[0]))
		{
			// 'fmt ' chunk
			case FCC('f','m','t',' '):
				// verifica size
				if(size > 100 || size < 16) return 0;

				// citeste continutul
				pf_read(BUF_FRONT, size, &ret);
				// verifica codificarea
				if(ret != size || BUF_FRONT[0] != 1) return 0;
				// verifica numarul de canale
				if(BUF_FRONT[2] != 1 && BUF_FRONT[2] != 2) return 0;
				// verifica rezolutia
				if(BUF_FRONT[14] != 8 && BUF_FRONT[14] != 16) return 0;

				// seteaza sampling rate-ul
				OCR0A = (BYTE)(F_CPU / 8 / LD_WORD(&BUF_FRONT[4])) - 1;
			//	PORTD ^= (1 << PD0);
				break;

			// 'data' chunk => incepe redarea
			case FCC('d','a','t','a'):
				return size;

			// 'LIST' chunk => skip
			case FCC('L','I','S','T'):
			// 'fact' chunk => skip
			case FCC('f','a','c','t'):
				pf_lseek(fs.fptr + size);
				break;

			// chunk necunoscut => eroare
			default:
				return 0;
		}
	}

	return 0;
}

/*
 * Functie care reda un fisier audio
 *
 * path - calea absoluta a fisierului
 *
 * @return UINT
 *	FR_OK daca a rulat cu succes fisierul
 */
UINT play(const char *path)
{
	FRESULT ret;

	if((ret = pf_open(path)) == FR_OK)
	{   
		WORD bytes_read;

		// incarca header-ul fisierului
		DWORD current_size = load_header();
		if(current_size < buf_size)
			return FR_NO_FILE;

		// align to sector boundary
		ret = pf_lseek((fs.fptr + 511) & ~511);
		if(ret != FR_OK)
			return ret;

		// fill front buffer
		ret = pf_read(BUF_FRONT, buf_size, &bytes_read);
		if(ret != FR_OK)
			return ret;
		if(bytes_read < buf_size)
			return ret;

		// reset front buffer index
		buf_pos = 0;

		// start output
		timer0_start();
		timer1_start();
		DDRD |= (1 << PD4);
		
		while(continue_play())
		{
			uint8_t 
			old_buf_front = buf_front;
			
			// fill back buffer
			ret = pf_read(BUF_BACK, buf_size, &bytes_read);
			if(ret != FR_OK)
				break;
			if(bytes_read < buf_size)
				break;
			// wait for buffer swap
			while(old_buf_front == buf_front) ;

		}

		// stop output
		DDRD &= ~(1 << PD4);
		timer1_stop();
		timer0_stop();
		_delay_ms(1000);
		LCD_printAt(0x40, "am terminat play");
	}

	return ret;
}


/*---------------------------------------------------------------------------*/
/* Ceas                                                                      */
/*---------------------------------------------------------------------------*/

volatile uint8_t hours = 0;
volatile uint8_t minutes = 0;
volatile uint8_t seconds = 0;


void get_music(int n, const char *folder, char *filename)
{
	DIR dir;
	FILINFO fil;

	char i = 0;

	pf_opendir(&dir, MUSIC);
	for(i = 0; i < n; i++) {
		pf_readdir(&dir, &fil);
		if(fil.fname[0] == 0x00) {
			filename[0] = 0x00;
			return;
		}
	}

	filename[0] = 0;
	strcat(filename, folder);
	strcat(filename, "/");
	strcat(filename, fil.fname);
}

void play_song(char file_no)
{
	DIR dir;
	if(pf_opendir(&dir, MUSIC) != FR_OK)
	{
		LCD_printAt(0x00, MUSIC"missing");
		return;
	}

	filename[0] = 0;
	get_music(file_no, MUSIC, filename);
	if (filename[0] == 0x00){
		file_no = 1;
		get_music(file_no, MUSIC, filename);
	}
	LCD_clear_top_line();
	LCD_printAt(0x00, filename);
	_delay_ms(1000);
	play(filename);

}


bool SensorOn(uint8_t nr)
{
	
	
	if (SensorsVals[nr] < 300)
	return true;
	
}


void verify()
{
	for(int i= 0; i<adc_num_channels; i++)
	SensorsVals[i] = 0;

	uint16_t a;

	for(int i= 0; i<adc_num_channels; i++)
	{
		const adc_channel_t *channel = &ADC_channels[i];
		LCD_clear_top_line();
		LCD_printAt(0x00, channel->name);
		a = ADC_get(channel->channel);
		SensorsVals[i] = a;
			
		int m = SensorsVals[i];
		char arr[200], arr2[200];
		sprintf(arr, "da: %d", m);
		sprintf(arr2, "da: %d", a);
		LCD_print(arr);
		LCD_printAt(0x40, arr2);
			
		_delay_ms(1000);
			
	}
	
	if(SensorsVals[1] < 300)
	play_song(1);
	
	if(SensorsVals[2] < 300)
	play_song(2);
	
	if(SensorsVals[3] < 300)
	play_song(3);
	
	if(SensorsVals[4] < 300)
	play_song(4);
	
	if(SensorsVals[5] < 300)
	play_song(5);
	
	
	

	
	
}

void task1()
{
	int i;
	if(!(PIND & (1 << PD6)))
	//for( i = 0; i < 2; i++)
	{	
		//LCD_clear_top_line();
		//LCD_printAt(0x00, "task1");
		_delay_ms(1000);
		
		LCD_clear_top_line();
		DIR dir;
		FILINFO fil;

		pf_opendir(&dir, MUSIC);
		do {
			pf_readdir(&dir, &fil);
			LCD_clear_top_line();
			LCD_printAt(0x00, fil.fname);
			_delay_ms(1000);
		} while(fil.fname[0] != 0x00);
	}
}


void task2and3()
{
	/* Conditia de declansare: PB2 low, PD6 low */
	if(!(PIND & (1 << PD6)))
	{
		LCD_clear_top_line();

		DIR dir;
		if(pf_opendir(&dir, MUSIC) != FR_OK)
		{
			LCD_printAt(0x00, MUSIC"missing");
			return;
		}
	}

	/* Conditia de declansare: PB2 low, PD6 high */
	if(!(PIND & (1 << PD6)))
	{
		filename[0] = 0;
		get_music(current_file_no, MUSIC, filename);
		if (filename[0] == 0x00){
			current_file_no = 1;
			get_music(current_file_no, MUSIC, filename);
		}

		LCD_clear_top_line();
		LCD_printAt(0x00, filename);
		current_file_no += 1;
		
		_delay_ms(1000);
		LCD_clear_top_line();
		LCD_printAt(0x00, filename);
		_delay_ms(1000);
		play(filename);
	
	}


}


void exemplu_lcd()
{
	if(((PIND & (1 << PD6)) == 0))
		LCD_print("Hello World!");
	//LCD_printAt(0x40, "mesaj linie 2");
    for(;;);
}


void testspeaker()
{
	DDRD |= (1 << PD4);

	while(1) {
	/* Invers?m starea pinului. */
	PORTD ^= (1 << PD4);
	
	_delay_ms(10);
	}
}


int main()
{	
	SD_DDR |= (1 << SD_SS);
	
	IO_init();
	LCD_init();
	SPI_init();
	TIMER2_init();
	ADC_init();
	

	
//	timer2_init();
#ifdef ADC_USE_IRQ
// Init Timer1
TIMER2_init();
#endif
sei();

/*
	for(;;)
	{
		play_song(1);
		//ADC_show();
		//LCD_printAt(0x00, "bla bla");
		//_delay_ms(1000);
		
	}
*/

	for(;;)
	{
		// mount filesystem
		LCD_printAt(0x00, "mounting...");

		if(pf_mount(&fs) != FR_OK)
		{
			// wait a while and retry
			_delay_ms(1000);
			continue;
		}

		LCD_clear_top_line();
		LCD_printAt(0x00, "mounted!");
		_delay_ms(1000);
		LCD_clear_top_line();

		for(;;)
		{
			verify();
			//play_song(1);
			//task2and3();
			//testspeaker();
			//_delay_ms(2000);
		}
	
	}
	
	
	
/*
	for(;;)
	{
		ADC_show();		
	}
	return 0;
*/

		
	
	
}

