﻿/*
 * lab1.c
*/
#include <avr/io.h>
#include <avr/interrupt.h>

#define F_CPU 16000000

#include <string.h>
#include <stdio.h>
#include <stdbool.h>

#include "lcd.h"
#include "usart.h"
#include "pff.h"
#include "spi.h"
#include "sd.h"
#include "pff.h"
#include <avr/io.h>
#include <util/delay.h>

#define MUSIC "music"

#define FCC(c1, c2, c3, c4) \
(((DWORD)(c4) << 24) + \
((DWORD)(c3) << 16) + \
(( WORD)(c2) <<  8) + \
(( BYTE)(c1) <<  0))


uint8_t	buf[2][256];	// wave output buffers (double buffering)
const	 uint16_t	buf_size = 256;	// front and back buffer sizes
volatile uint8_t	buf_front = 0;	// front buffer index (current buffer used)
volatile uint8_t	buf_pos = 0;	// current buffer position
volatile uint8_t	buf_sync = 0;

#define BUF_FRONT	(buf[buf_front])
#define BUF_BACK	(buf[1 - buf_front])

char current_file_no = 1;
char filename[32] = { 0 };
FATFS fs;					// sistemul de fisiere



//************************************************************************************//
//                                       Initializare IO
//************************************************************************************//


void IO_init(void)
{
	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);
	
	DDRD |= (1 << PD0);
	DDRD |= (1 << PD1);
	
	DDRB &= ~(1<<PB0);
	PORTB |= (1 << PB0);
	
}

//************************************************************************************//
//                                         ADC
//************************************************************************************//

#define ADC_USE_IRQ

/* Tensiunea de referinta utilizata de ADC. */
#define ADC_AREF_VOLTAGE 3.3
#define ADC_MAX_CHANNELS 6

/* vector cu valorile senzorilor */
uint16_t SensorsVals[5];

/* Numele canalelor de ADC. */
typedef struct
{
	const char *name;
	uint8_t    channel;
} adc_channel_t;


/* Perechi de (Nume canal, Numar canal). */
const adc_channel_t ADC_channels[] = {
	{"Senzor1", 0},
    {"Senzor2", 1},
	{"Senzor3", 2},
	{"Senzor4", 3},
	{"Senzor5", 4},
};


/* Numarul de canale ADC utilizate. */
const int adc_num_channels = sizeof(ADC_channels) / sizeof(ADC_channels[0]);

#ifdef ADC_USE_IRQ
/* Numarul canalului initial. */
#define ADC_INIT_CHANNEL 0

/* Valorile citite de la fiecare canal. */
volatile uint16_t ADC_value[ADC_MAX_CHANNELS];
#endif

#ifdef ADC_USE_IRQ
ISR(ADC_vect)
{
    static uint8_t channel = ADC_INIT_CHANNEL;

    ADC_value[channel] = ADC;

    channel = (channel + 1) % ADC_MAX_CHANNELS;

    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    // Pornim ADC-ul daca le-a citit pe toate
    if (channel != ADC_INIT_CHANNEL)
        ADCSRA |= (1 << ADSC);
}

ISR(TIMER2_COMPA_vect)
{
	// TODO 4
	// start an ADC conversion
	ADCSRA |= (1 << ADSC);
}
#endif


/*
 * Functia initializeaza convertorul Analog-Digital.
 */
void ADC_init(void)
{

	ADMUX = (1 << REFS0) | (1 << ADC_INIT_CHANNEL);
	ADCSRA = (1 << ADEN) | (1 << ADIE) | (7 << ADPS0);
}

 //din laborator - preia datele de pe ADC
uint16_t ADC_get(uint8_t channel)
{
	#ifndef ADC_USE_IRQ
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while((ADCSRA & (1 << ADIF)) == 0);

	return ADC;

	#else
	return ADC_value[channel];

	#endif

(void)channel;
return 0; 
}


//Initializare timer 2 - pentru ADC - masoara la fiecare 100ms
#ifdef ADC_USE_IRQ
void TIMER2_init()
{
	TCCR2A |= (1 << WGM21);
	TCCR2B |= (7 << CS20);
	TIMSK2 |= (1 << OCIE2A);
	OCR2A = 1561;
}
#endif

//******************************************************************//
//                            Speaker
//******************************************************************//


//Rutina de tratare a intreruperii pentru timer0
ISR(TIMER0_COMPA_vect)
{
	
	OCR1B = BUF_FRONT[buf_pos++];
	if(buf_pos == 0)
		buf_front = 1 - buf_front;

		
}

//Initializari timere 
void timer0_start(void)
{
	// interrupt on compare A
	TIMSK0 |= (1 << OCIE0A);
	// CTC, top OCRA
	TCCR0B |= (0 << WGM02);
	TCCR0A |= (1 << WGM01) | (0 << WGM00);
	// prescaler 8
	TCCR0B |= (2 << CS00);
}

void timer0_stop(void)
{
	TCCR0B = 0;
	TCCR0A = 0;
	TIMSK0 = 0;
	OCR0A = 0;
	TCNT0 = 0;
}

void timer1_start(void)
{
	// 8-bit FastPWM
	TCCR1B |= (1 << WGM12);
	TCCR1A |= (1 << WGM10);
	// channel B inverted
	TCCR1A |= (1 << COM1B0) | (1 << COM1B1);
	// prescaler 1
	TCCR1B |= (1 << CS10);
}

void timer1_stop(void)
{
	TCCR1B = 0;
	TCCR1A = 0;
	OCR1B = 0;
	TCNT1 = 0;
}

//Functie din laborator
bool continue_play()
{
	if((PINB & (1 << PB6)) == 0)
		return false;
	else
	return true;
}


//Functie din laborator
/*
 * Incarca header-ul unui fisier WAVE
 *
 * @return DWORD
 * 	0 => format invalid
 * 	1 => eroare I/O
 * 	>1 => numarul de sample-uri
 */
DWORD load_header(void)
{
	DWORD size;
	WORD ret;

	// citeste header-ul (12 octeti)
	if(pf_read(BUF_FRONT, 12, &ret))
		return 1;

	if(ret != 12 || LD_DWORD(BUF_FRONT + 8) != FCC('W','A','V','E'))
		return 0;

	for(;;)
	{
		// citeste chunk ID si size
		pf_read(BUF_FRONT, 8, &ret);
		if(ret != 8)
			return 0;

		size = LD_DWORD(&BUF_FRONT[4]);

		// verifica FCC
		switch(LD_DWORD(&BUF_FRONT[0]))
		{
			// 'fmt ' chunk
			case FCC('f','m','t',' '):
				// verifica size
				if(size > 100 || size < 16) return 0;

				// citeste continutul
				pf_read(BUF_FRONT, size, &ret);
				// verifica codificarea
				if(ret != size || BUF_FRONT[0] != 1) return 0;
				// verifica numarul de canale
				if(BUF_FRONT[2] != 1 && BUF_FRONT[2] != 2) return 0;
				// verifica rezolutia
				if(BUF_FRONT[14] != 8 && BUF_FRONT[14] != 16) return 0;

				// seteaza sampling rate-ul
				OCR0A = (BYTE)(F_CPU / 8 / LD_WORD(&BUF_FRONT[4])) - 1;
			//	PORTD ^= (1 << PD0);
				break;

			// 'data' chunk => incepe redarea
			case FCC('d','a','t','a'):
				return size;

			// 'LIST' chunk => skip
			case FCC('L','I','S','T'):
			// 'fact' chunk => skip
			case FCC('f','a','c','t'):
				pf_lseek(fs.fptr + size);
				break;

			// chunk necunoscut => eroare
			default:
				return 0;
		}
	}

	return 0;
}

//Functie din laborator
UINT play(const char *path)
{
	FRESULT ret;

	if((ret = pf_open(path)) == FR_OK)
	{   
		WORD bytes_read;

		// incarca header-ul fisierului
		DWORD current_size = load_header();
		if(current_size < buf_size)
			return FR_NO_FILE;

		// align to sector boundary
		ret = pf_lseek((fs.fptr + 511) & ~511);
		if(ret != FR_OK)
			return ret;

		// fill front buffer
		ret = pf_read(BUF_FRONT, buf_size, &bytes_read);
		if(ret != FR_OK)
			return ret;
		if(bytes_read < buf_size)
			return ret;

		// reset front buffer index
		buf_pos = 0;

		// start output
		timer0_start();
		timer1_start();
		DDRD |= (1 << PD4);
		
		while(continue_play())
		{
			uint8_t 
			old_buf_front = buf_front;
			
			// fill back buffer
			ret = pf_read(BUF_BACK, buf_size, &bytes_read);
			if(ret != FR_OK)
				break;
			if(bytes_read < buf_size)
				break;
			// wait for buffer swap
			while(old_buf_front == buf_front) ;
		}

		// stop output
		DDRD &= ~(1 << PD4);
		timer1_stop();
		timer0_stop();
		_delay_ms(1000);
	}

	return ret;
}


void get_music(int n, const char *folder, char *filename)
{
	DIR dir;
	FILINFO fil;

	char i = 0;

	pf_opendir(&dir, MUSIC);
	for(i = 0; i < n; i++) {
		pf_readdir(&dir, &fil);
		if(fil.fname[0] == 0x00) {
			filename[0] = 0x00;
			return;
		}
	}

	filename[0] = 0;
	strcat(filename, folder);
	strcat(filename, "/");
	strcat(filename, fil.fname);
}

void play_song(char file_no)
{
	DIR dir;
	if(pf_opendir(&dir, MUSIC) != FR_OK)
	{
		LCD_printAt(0x00, MUSIC"missing");
		return;
	}

	filename[0] = 0;
	get_music(file_no, MUSIC, filename);
	if (filename[0] == 0x00){
		file_no = 1;
		get_music(file_no, MUSIC, filename);
	}
	play(filename);

}

//Selecteaza cazul ce va fi utilizat si trateaza ca atare (afiseaza pe lcd si audiaza)
void ShowSpeak(uint16_t no)
{ 
	LCD_clear_top_line();
	LCD_clear_bottom_line();
	_delay_ms(40);
	
	switch(no)
	{
		case 1: 
			LCD_printAt(0x00, "The check,");
			LCD_printAt(0x40, "please!"); 
		break;
		
		case 2:
			LCD_printAt(0x00, "Doctor!");
		break;
		
		case 3:
			LCD_printAt(0x00, "I'm lost!");
		break;
		
		case 4:
			LCD_printAt(0x00, "Could you recomm");
			LCD_printAt(0x40, "end a restaurant?!");
		break;
		
		case 5:
			LCD_printAt(0x00, "Thank you!");
		break;
		
		
		case 6:
			LCD_printAt(0x00, "Water!");
		break;
		
		case 7:
			LCD_printAt(0x00, "Where is the");
			LCD_printAt(0x40, "bathroom?");
		break;

	}
	
	_delay_ms(1);
	play_song(no);		
	LCD_clear_top_line();
	LCD_clear_bottom_line();
	_delay_ms(40);
}


//Verifica valorile date de senzori si alege pe care caz va merge
void verify()
{
	for(int i= 0; i<adc_num_channels; i++)
	SensorsVals[i] = 0;

	uint16_t a;

	for(int i= 0; i<adc_num_channels; i++)
	{
		const adc_channel_t *channel = &ADC_channels[i];	
		a = ADC_get(channel->channel);
		SensorsVals[i] = a;
		_delay_ms(10);
			
	}
	
	if( (SensorsVals[4] < 300)  && (SensorsVals[3] < 300) )
		ShowSpeak(7);
	else 
		if( (SensorsVals[1] < 300) && (SensorsVals[4] < 300) )
			ShowSpeak(6);
		else
			if(SensorsVals[1] < 300)
				ShowSpeak(1);
			else
				if(SensorsVals[1] < 300)
					ShowSpeak(1);
				else
					if(SensorsVals[2] < 300)
					ShowSpeak(2);
					else
						if(SensorsVals[3] < 300)
							ShowSpeak(3);
						else
							if(SensorsVals[4] < 300)
								ShowSpeak(4);
							else
								if(SensorsVals[5] < 300)
									ShowSpeak(5);

}


int main()
{	
	SD_DDR |= (1 << SD_SS);
	
	IO_init();
	LCD_init();
	SPI_init();
	TIMER2_init();
	ADC_init();
	
	#ifdef ADC_USE_IRQ
	TIMER2_init();
	#endif
	
	sei();

	
	// mount 
	LCD_printAt(0x00, "mounting...");

	if(pf_mount(&fs) != FR_OK)
	{
		_delay_ms(1000);
		
	}

	LCD_clear_top_line();
	LCD_printAt(0x00, "mounted!");
	_delay_ms(1000);
	
	for(;;)
	{ 
		
		LCD_clear_top_line();
		LCD_printAt(0x00, "ready!");
		LCD_clear_top_line();
		_delay_ms(500);
			
		if( (PINB & (1 << PB0)) == 0)
			{
				verify();
			}
			
		else
			{
				LCD_clear_top_line();
				LCD_printAt(0x00,"horizontal hand ");
				LCD_printAt(0x40,"position needed");
				_delay_ms(500);

			}

	}

	
}

